#!/usr/bin/env node
'use strict';

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _subarg = require('subarg');

var _subarg2 = _interopRequireDefault(_subarg);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _validFile = require('valid-file');

var _validFile2 = _interopRequireDefault(_validFile);

var _dist = require('./../dist/');

var _dist2 = _interopRequireDefault(_dist);

var cli = (0, _meow2['default'])('\n\n  Get timecard setup with the \'new\' command, then use the \'clockin\' and \'clockout\' commands\n  to record your time. When you want to see a summary of your time, use the \'print\' command.\n\n  Commands\n    timecard new            Setup a blank timecard for your project\n    timecard clockin        Set the start time for your shift\n    timecard clockout       Set the end time for your shift\n    timecard print          Print a summary of your time\n\n  Options\n    -h,  --help             Show this help message\n    -v,  --version          Show the current timecard version\n    -i,  --clockin          Alias for the clockin command\n    -o,  --clockout         Alias for the clockout command\n    -n,  --new              Alias for the new command\n    -p,  --print            Alias for the print command\n    --no-prompt             Use with \'new\' to silence all prompts and overwrite existing timecard\n    --message, -m           Include a message summary to your shift (via clockin and clockout)\n\n', {
	alias: {
		h: 'help',
		v: 'version',
		i: 'clockin',
		o: 'clockout',
		p: 'print',
		n: 'new',
		m: 'message'
	}
});

function init(args, options) {
	var _cmd, defaults, timecard, timeCardExists, timecardData, timeCardValid, message, command;

	return _regeneratorRuntime.async(function init$(context$1$0) {
		while (1) switch (context$1$0.prev = context$1$0.next) {
			case 0:
				if (args.length === 0 && _Object$keys(options).length === 0) {
					cli.showHelp(1);
				}

				_cmd = {
					is: function is(cmd) {
						return args.indexOf(cmd) > -1 || options[cmd];
					},
					not: function not(cmd) {
						return args.indexOf(cmd) === -1 && !options[cmd];
					}
				};
				defaults = {
					name: cli.pkg.name,
					prompt: true
				};

				options = (0, _objectAssign2['default'])(defaults, options);

				timecard = new _dist2['default'](options);
				timeCardExists = _validFile2['default'].sync(timecard.filepath);
				context$1$0.next = 8;
				return _regeneratorRuntime.awrap(timecard.getTimeCard(timecard.filepath));

			case 8:
				timecardData = context$1$0.sent;
				context$1$0.next = 11;
				return _regeneratorRuntime.awrap(timecard.isTimeCardValid(timecardData));

			case 11:
				timeCardValid = context$1$0.sent;

				if (timeCardExists && timecardData && !timeCardValid) {
					console.log(timecard.errors.invalidTimeCardJSON);
					process.exit(1);
				}

				// If a timecard exists, load its data into the Timecard instance.

				if (!_cmd.not('new')) {
					context$1$0.next = 17;
					break;
				}

				if (!timeCardExists) {
					context$1$0.next = 17;
					break;
				}

				context$1$0.next = 17;
				return _regeneratorRuntime.awrap(timecard.load(timecardData));

			case 17:
				message = options.message || options.m;
				command = undefined;

				if (_cmd.is('new')) {
					command = timecard.create;
				}

				if (_cmd.is('clockin')) {
					command = timecard.clockin;
				}

				if (_cmd.is('clockout')) {
					command = timecard.clockout;
				}

				if (_cmd.is('print')) {
					command = timecard.print;
				}

				// Now we are clear to call the API.
				command.call(timecard, message).then(function (response) {
					console.log(response);
				})['catch'](function (err) {
					console.log(err);
				});

			case 24:
			case 'end':
				return context$1$0.stop();
		}
	}, null, this);
}

(0, _updateNotifier2['default'])({ pkg: cli.pkg }).notify();

init((0, _subarg2['default'])(cli.input)._, cli.flags);

// Do we have a message in this command call?