'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _findUp = require('find-up');

var _findUp2 = _interopRequireDefault(_findUp);

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _convertSeconds = require('convert-seconds');

var _convertSeconds2 = _interopRequireDefault(_convertSeconds);

var _validFile = require('valid-file');

var _validFile2 = _interopRequireDefault(_validFile);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _pendel = require('pendel');

var _pendel2 = _interopRequireDefault(_pendel);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _promptsJs = require('./prompts.js');

var TimeCard = (function () {
  /**
   * Initialize a new Timecard
   *
   * @param {Object} options
   * @api public
   */

  function TimeCard(options) {
    _classCallCheck(this, TimeCard);

    this.options = (0, _objectAssign2['default'])({}, options);

    var pkgRoot = _findUp2['default'].sync('package.json');
    var root = pkgRoot ? _path2['default'].dirname(pkgRoot) : process.cwd();
    this.filepath = this.options.filepath ? this.options.filepath : _path2['default'].join(root, '.timecard.json');
    this.name = this.options.name || this.filepath;
    this.prompt = typeof this.options.prompt === 'boolean' ? this.options.prompt : true;

    // Timecard state.
    this.pendingClockoutIndex = undefined;
    this.totalSeconds = 0;
    this.shiftOrder = [];
    this.shifts = {};
  }

  /**
   * Use existing Timecard data to populate the Timecard's state.
   *
   * @param {object} timecard - the timecard data in JSON format.
   * @returns {*} Promise
   */

  _createClass(TimeCard, [{
    key: 'load',
    value: function load(timecard) {
      var self;
      return _regeneratorRuntime.async(function load$(context$2$0) {
        var _this = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            self = this;
            return context$2$0.abrupt('return', new _Promise(function callee$2$0(resolve, reject) {
              var _timecard;

              return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                while (1) switch (context$3$0.prev = context$3$0.next) {
                  case 0:
                    _timecard = undefined;

                    if (typeof timecard === 'object') {
                      _timecard = timecard;
                    } else if (typeof timecard === 'string') {
                      try {
                        _timecard = JSON.parse(timecard);
                      } catch (err) {
                        reject(err);
                      }
                    }

                    self.name = _timecard.project;
                    self.shifts = _timecard.shifts;
                    self.totals = _timecard.totals;
                    self.shiftOrder = self.getShiftOrder(self.shifts);
                    self.totalSeconds = self.getTotalSeconds(self.shifts);
                    self.pendingClockoutIndex = self.getPendingClockoutId(self.shifts);
                    resolve(self);

                  case 9:
                  case 'end':
                    return context$3$0.stop();
                }
              }, null, _this);
            }));

          case 2:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }

    /**
     * Create a blank timecard on `timecard new`
     *
     */
  }, {
    key: 'create',
    value: function create() {
      var self, msg;
      return _regeneratorRuntime.async(function create$(context$2$0) {
        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            self = this;
            msg = _chalk2['default'].gray(self.filepath) + ' \n ' + self.messages.createdNewTimeCard;
            return context$2$0.abrupt('return', new _Promise(function (resolve, reject) {
              if (self.prompt === true) {
                var questions = (0, _promptsJs.createPrompt)();

                // First check to see if the file already exists
                if (_validFile2['default'].sync(self.filepath) === true) {
                  questions = (0, _promptsJs.eraseThenCreatePrompt)();
                }

                _inquirer2['default'].prompt(questions, function (answers) {
                  if (typeof answers.eraseCard === 'undefined' || answers.eraseCard === true) {
                    self.writeTimeCard(self.filepath, self.blankTimecard(answers.projectName)).then(function () {
                      resolve(msg);
                    });
                  }
                });
              } else {
                self.writeTimeCard(self.filepath, self.blankTimecard(self.name)).then(function () {
                  resolve(msg);
                })['catch'](function (err) {
                  reject(err);
                });
              }
            }));

          case 3:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }

    /**
     * Clockin.
     *
     * @param {string} _message - an optional shift message
     */
  }, {
    key: 'clockin',
    value: function clockin(_message) {
      var self;
      return _regeneratorRuntime.async(function clockin$(context$2$0) {
        var _this2 = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            self = this;
            return context$2$0.abrupt('return', new _Promise(function callee$2$0(resolve, reject) {
              var newShift, nextShiftId, timecardString;
              return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                while (1) switch (context$3$0.prev = context$3$0.next) {
                  case 0:
                    // Don't try to clockin if a clockout is pending
                    if (self.pendingClockoutIndex) {
                      reject(self.errors.clockOutIsPending);
                    } else {
                      newShift = self.getNewShift(_message);
                      nextShiftId = self.getNextShiftId(self);

                      self.pendingClockoutIndex = nextShiftId;
                      self.shiftOrder.push(nextShiftId);
                      self.shifts[nextShiftId] = newShift;

                      timecardString = self.getTimecardString(self);

                      self.writeTimeCard(self.filepath, timecardString).then(function () {
                        resolve(self.messages.successfulClockin);
                      })['catch'](function (err) {
                        reject(err);
                      });
                    }

                  case 1:
                  case 'end':
                    return context$3$0.stop();
                }
              }, null, _this2);
            }));

          case 2:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }

    /**
     * Clockout.
     *
     * @param {string} _message - an optional shift message
     */
  }, {
    key: 'clockout',
    value: function clockout(_message) {
      var self;
      return _regeneratorRuntime.async(function clockout$(context$2$0) {
        var _this3 = this;

        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            self = this;
            return context$2$0.abrupt('return', new _Promise(function callee$2$0(resolve, reject) {
              return _regeneratorRuntime.async(function callee$2$0$(context$3$0) {
                while (1) switch (context$3$0.prev = context$3$0.next) {
                  case 0:
                    if (_validFile2['default'].sync(self.filepath) === false) {
                      reject(self.errors.noTimeCardFoundForClockout);
                    } else if (typeof self.pendingClockoutIndex === 'undefined') {
                      reject(self.errors.noClockInFound);
                    } else {
                      (function () {
                        var date = new Date().toString();

                        var shiftId = self.pendingClockoutIndex;
                        var shift = self.shifts[shiftId];

                        var message = self.stringOrUndefined(_message);

                        if (message) {
                          shift.messages.push(message);
                        }

                        shift.endTime = date;

                        // Update self.shift object with the new shift data
                        self.shifts[shiftId] = shift;

                        // Report the last 'shift' time and the total time to the console.
                        var shiftSeconds = _pendel2['default'].date(shift.startTime, shift.endTime).seconds;
                        var clockOutSummary = self.clockoutSummary(shiftSeconds, self.totalSeconds + shiftSeconds);

                        self.totalSeconds += shiftSeconds;
                        self.totals = (0, _convertSeconds2['default'])(self.totalSeconds);

                        var timecardString = self.getTimecardString(self);

                        self.writeTimeCard(self.filepath, timecardString).then(function () {
                          resolve(clockOutSummary + '\n ' + self.messages.successfulClockout);
                        })['catch'](function (err) {
                          reject(err);
                        });
                      })();
                    }

                  case 1:
                  case 'end':
                    return context$3$0.stop();
                }
              }, null, _this3);
            }));

          case 2:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }

    /**
     * Print the timecard data to the console.
     *
     */
  }, {
    key: 'print',
    value: function print() {
      var self, output;
      return _regeneratorRuntime.async(function print$(context$2$0) {
        while (1) switch (context$2$0.prev = context$2$0.next) {
          case 0:
            self = this;
            output = self.prettyPrintHeader();

            self.shiftOrder.forEach(function (shiftId) {
              var shift = self.shifts[shiftId];
              output += self.prettyPrintEntry(shiftId, shift);
            });
            output += self.messages.prettyPrintBorder;
            output += self.summary(self.totalSeconds);
            return context$2$0.abrupt('return', _Promise.resolve(output));

          case 6:
          case 'end':
            return context$2$0.stop();
        }
      }, null, this);
    }
  }]);

  return TimeCard;
})();

exports['default'] = TimeCard;

(0, _objectAssign2['default'])(TimeCard.prototype, require('./utils.js'));
(0, _objectAssign2['default'])(TimeCard.prototype, require('./messages.js'));
module.exports = exports['default'];